@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -47,8 +47,8 @@
 #include "linebuffer.h"
 #define Version VERSION
 #include "version-etc.h"
-#include "xstrndup.h"
 #include "xalloc.h"
+#include "sh-quote.h"
 
 #include "text-options.h"
 #include "text-lines.h"
@@ -62,10 +62,12 @@
 /* The official name of this program (e.g., no 'g' prefix).  */
 #define PROGRAM_NAME "datamash"
 
-#define AUTHORS proper_name ("Assaf Gordon")
+#define AUTHORS proper_name \
+  ("Assaf Gordon, Tim Rice, Shawn Wagner, Erik Auerswald")
 
 /* Until someone better comes along */
-const char version_etc_copyright[] = "Copyright %s %d Assaf Gordon" ;
+const char version_etc_copyright[]
+  = "Copyright %s %d Assaf Gordon and Tim Rice" ;
 
 /* Line number in the input file */
 static size_t line_number = 0 ;
@@ -100,6 +102,9 @@ static size_t rmdup_initial_size = (1024*1024);
 /* Explicit output delimiter with --output-delimiter */
 static int explicit_output_delimiter = -1;
 
+/* Path to default sort program */
+static const char *sort_cmd = SORT_PATH;
+
 enum
 {
   INPUT_HEADER_OPTION = CHAR_MAX + 1,
@@ -108,13 +113,14 @@ enum
   REMOVE_NA_VALUES_OPTION,
   OUTPUT_DELIMITER_OPTION,
   CUSTOM_FORMAT_OPTION,
+  SORT_PROGRAM_OPTION,
   UNDOC_PRINT_INF_OPTION,
   UNDOC_PRINT_NAN_OPTION,
   UNDOC_PRINT_PROGNAME_OPTION,
   UNDOC_RMDUP_TEST
 };
 
-static char const short_options[] = "sfF:izg:t:HWR:C";
+static char const short_options[] = "sfF:izg:t:HWR:Cc:";
 
 static struct option const long_options[] =
 {
@@ -131,10 +137,12 @@ static struct option const long_options[] =
   {"filler", required_argument, NULL, 'F'},
   {"format", required_argument, NULL, CUSTOM_FORMAT_OPTION},
   {"output-delimiter", required_argument, NULL, OUTPUT_DELIMITER_OPTION},
+  {"collapse-delimiter", required_argument, NULL,'c'},
   {"sort", no_argument, NULL, 's'},
   {"no-strict", no_argument, NULL, NO_STRICT_OPTION},
   {"narm", no_argument, NULL, REMOVE_NA_VALUES_OPTION},
   {"round", required_argument, NULL, 'R'},
+  {"sort-cmd", required_argument, NULL, SORT_PROGRAM_OPTION},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
   /* Undocumented options */
@@ -210,12 +218,14 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs (_("Statistical Grouping operations:\n"),stdout);
       fputs ("\
   mean, geomean, harmmean, trimmean, median, q1, q3, iqr, perc,\n\
-  mode, antimode, pstdev, sstdev, pvar, svar, mad, madraw,\n\
+  mode, antimode, pstdev, sstdev, pvar, svar, ms, rms, mad, madraw,\n\
   pskew, sskew, pkurt, skurt, dpo, jarque,\n\
   scov, pcov, spearson, ppearson\n\
 \n", stdout);
       fputs ("\n", stdout);
 
+      fputs (_("Options:\n"),stdout);
+      fputs ("\n", stdout);
       fputs (_("Grouping Options:\n"),stdout);
       fputs (_("\
   -C, --skip-comments       skip comment lines (starting with '#' or ';'\n\
@@ -224,6 +234,10 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs (_("\
   -f, --full                print entire input line before op results\n\
                               (default: print only the grouped keys)\n\
+                            This option is only sensible for linewise\n\
+                            operations. Other uses are deprecated and\n\
+                            will be removed in a future version of GNU\n\
+                            Datamash.\n\
 "), stdout);
       fputs (_("\
   -g, --group=X[,Y,Z]       group via fields X,[Y,Z];\n\
@@ -246,14 +260,18 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
   -s, --sort                sort the input before grouping; this removes the\n\
                               need to manually pipe the input through 'sort'\n\
 "), stdout);
+      fputs (_("\
+  -c, --collapse-delimiter=X  use X to separate elements in collapse and\n\
+                              unique lists (default: comma)\n\
+"), stdout);
 
       fputs (_("File Operation Options:\n"),stdout);
       fputs (_("\
       --no-strict           allow lines with varying number of fields\n\
 "), stdout);
-      fputs (_("\
+      printf (_("\
       --filler=X            fill missing values with X (default %s)\n\
-"), stdout);
+"), missing_field_filler);
 
       fputs ("\n", stdout);
       fputs (_("General Options:\n"),stdout);
@@ -281,12 +299,23 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs (_("\
   -z, --zero-terminated     end lines with 0 byte, not newline\n\
 "), stdout);
+      fputs (_("\
+      --sort-cmd=/path/to/sort   Alternative sort(1) to use.\n\
+"), stdout);
 
       fputs (HELP_OPTION_DESCRIPTION, stdout);
       fputs (VERSION_OPTION_DESCRIPTION, stdout);
 
       fputs ("\n\n", stdout);
 
+      fputs (_("Environment:"), stdout);
+      fputs ("\n", stdout);
+      fputs (_("\
+  LC_NUMERIC        decimal-point character and thousands separator\n\
+"), stdout);
+
+      fputs ("\n\n", stdout);
+
       fputs (_("Examples:"), stdout);
       fputs ("\n\n", stdout);
       fputs (_("Print the sum and the mean of values from column 1:"), stdout);
@@ -302,7 +331,7 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
 \n", program_name);
 
       fputs (_("For detailed usage information and examples, see\n"),stdout);
-      printf ("  man %s\n", PACKAGE_NAME);
+      printf ("  man %s\n", PROGRAM_NAME);
       fputs (_("The manual and more examples are available at\n"), stdout);
       fputs ("  " PACKAGE_URL "\n\n", stdout);
     }
@@ -332,7 +361,7 @@ safe_line_record_get_fieldz (const struct line_record_t *lr, const size_t n,
                              char * /* out */ buf, size_t len)
 {
   const char *p;
-  size_t l;
+  size_t l = 0;
   safe_line_record_get_field (lr, n, &p, &l);
   l = MIN (len-1,l);
   memcpy (buf, p, l);
@@ -421,14 +450,14 @@ print_input_line (const struct line_record_t* lb)
     }
 }
 
-#define SWAP_LINES(A, B)			\
-  do						\
-    {						\
-      struct line_record_t *_tmp;		\
-      _tmp = (A);				\
-      (A) = (B);				\
-      (B) = _tmp;				\
-    }						\
+#define SWAP_LINES(A, B)          \
+  do                              \
+    {                             \
+      struct line_record_t *_tmp; \
+      _tmp = (A);                 \
+      (A) = (B);                  \
+      (B) = _tmp;                 \
+    }                             \
   while (0)
 
 
@@ -465,8 +494,6 @@ print_column_headers ()
   for (size_t i=0; i<dm->num_ops; ++i)
     {
       struct fieldop *op = &dm->ops[i];
-      if (op->slave)
-        continue;
 
       if (op->field > get_num_column_headers ())
         error_not_enough_fields (op->field, get_num_column_headers ());
@@ -480,7 +507,15 @@ print_column_headers ()
         printf (":%Lg", op->params.trimmed_mean);
       }
 
-      printf ("(%s)", get_input_field_name (op->field));
+      printf ("(%s", get_input_field_name (op->field));
+      while (dm->ops[i].slave)
+        {
+          /* print subsequent arguments to the same operation,
+             e.g. 'pcov (x,y)' */
+          ++i;
+          printf (",%s", get_input_field_name (dm->ops[i].field));
+        }
+      printf (")");
 
       if (i != dm->num_ops-1)
         print_field_separator ();
@@ -597,7 +632,7 @@ process_group (const struct line_record_t* line)
 /*
     Process each line in the input.
 
-    If the key fo the current line is different from the prevopis one,
+    If the key of the current line is different from the previous one,
     summarize the previous group and start a new one.
  */
 static void
@@ -617,6 +652,10 @@ process_file ()
   if (input_header && line_number==0)
     process_input_header (input_stream);
 
+  if (print_full_line && !line_mode)
+    fputs (_("datamash: Using -f/--full with non-linewise operations \
+is deprecated and will be disabled in a future release.\n"), stderr);
+
   /* If there is an input header line, and the user requested an output
      header line, and the input line was read successfully, print headers */
   if (input_header && output_header && line_number==1)
@@ -987,9 +1026,18 @@ remove_dups_in_file ()
 
           if (output_header)
             {
-              ignore_value (fwrite (line_record_buffer (thisline),
-                                    line_record_length (thisline),
-                                    sizeof (char), stdout));
+              const size_t num_fields = line_record_num_fields (thisline);
+              for (size_t i = 1 ; i <= num_fields ; ++i) {
+                if (i>1)
+                  print_field_separator ();
+
+                const char *str;
+                size_t len;
+                if (line_record_get_field (thisline, i, &str, &len))
+                  {
+                    ignore_value (fwrite (str, len, sizeof (char), stdout));
+                  }
+              }
               print_line_separator ();
             }
         }
@@ -1007,39 +1055,48 @@ remove_dups_in_file ()
         break;
       line_number++;
 
-       if (!line_record_get_field (thisline, key_col, &str, &len))
-         error_not_enough_fields (key_col, line_record_num_fields (thisline));
+      if (!line_record_get_field (thisline, key_col, &str, &len))
+        error_not_enough_fields (key_col, line_record_num_fields (thisline));
 
-       /* Add key to the key buffer */
-       if (next_key_pos + len + 1 > keys_buffer_alloc)
-         {
-           keys_buffer = xmalloc ( keys_buffer_alloc ) ;
-           next_key_pos = 0;
+      /* Add key to the key buffer */
+      if (next_key_pos + len + 1 > keys_buffer_alloc)
+        {
+          keys_buffer = xmalloc ( keys_buffer_alloc ) ;
+          next_key_pos = 0;
 
-           /* Add new key-buffer to the list */
-           if (buffer_list_size == buffer_list_alloc)
-              buffer_list = x2nrealloc (buffer_list,
-                                        &buffer_list_alloc, sizeof (char*));
-           buffer_list[buffer_list_size++] = keys_buffer;
-         }
-       char *next_key = keys_buffer+next_key_pos;
-       memcpy (next_key, str, len);
-       next_key[len] = 0;
+          /* Add new key-buffer to the list */
+          if (buffer_list_size == buffer_list_alloc)
+             buffer_list = x2nrealloc (buffer_list,
+                                       &buffer_list_alloc, sizeof (char*));
+          buffer_list[buffer_list_size++] = keys_buffer;
+        }
+      char *next_key = keys_buffer+next_key_pos;
+      memcpy (next_key, str, len);
+      next_key[len] = 0;
 
-       /* Add key to buffer (if not found) */
-       const int i = hash_insert_if_absent (ht, next_key, NULL);
-       if ( i == -1 )
-         die (EXIT_FAILURE, errno, _("hash memory allocation error"));
+      /* Add key to buffer (if not found) */
+      const int i = hash_insert_if_absent (ht, next_key, NULL);
+      if ( i == -1 )
+        die (EXIT_FAILURE, errno, _("hash memory allocation error"));
 
-       if ( i == 1 )
-         {
-           /* This string was not found in the hash - new key */
-           next_key_pos += len+1;
-           ignore_value (fwrite (line_record_buffer (thisline),
-                                 line_record_length (thisline), sizeof (char),
-                                 stdout));
-           print_line_separator ();
-         }
+      if ( i == 1 )
+        {
+          /* This string was not found in the hash - new key */
+          next_key_pos += len+1;
+          const size_t num_fields = line_record_num_fields (thisline);
+          for (size_t i = 1 ; i <= num_fields ; ++i) {
+            if (i>1)
+              print_field_separator ();
+
+            const char *str;
+            size_t len;
+            if (line_record_get_field (thisline, i, &str, &len))
+              {
+                ignore_value (fwrite (str, len, sizeof (char), stdout));
+              }
+          }
+          print_line_separator ();
+        }
     }
   line_record_free (&lr);
   hash_free (ht);
@@ -1054,9 +1111,9 @@ open_input ()
 {
   if (pipe_through_sort && dm->num_grps>0)
     {
-      char tmp[INT_BUFSIZE_BOUND (size_t)*2+5];
-      char cmd[1024];
-      memset (cmd,0,sizeof (cmd));
+      char delim[2] = { 0, 0 };
+      char **args = xcalloc (dm->num_grps + 8, sizeof (char *));
+      int argc = 0;
 
       if (input_header)
         {
@@ -1066,41 +1123,60 @@ open_input ()
              the 'sort' child-process */
           process_input_header (stdin);
         }
+
 #ifdef SORT_WITHOUT_LOCALE
-      /* For mingw/windows systems */
-      strcat (cmd,"sort ");
-#else
-      strcat (cmd,"LC_ALL=C sort ");
+      args[argc++] = "LC_ALL=C";
 #endif
-      if (!case_sensitive)
-        strcat (cmd,"-f ");
+      args[argc++] = (char *)sort_cmd;
 #ifdef HAVE_STABLE_SORT
       /* stable sort (-s) is needed to support first/last operations
          (prevent sort from re-ordering lines which are not part of the group.
          '-s' is not standard POSIX, but very commonly supported, including
          on GNU coreutils, Busybox, FreeBSD, MacOSX */
-      strcat (cmd,"-s ");
+      args[argc++] = "-s";
 #endif
-      if (in_tab != TAB_WHITESPACE)
+
+      if (eolchar == '\0')
         {
-          /* If the delimiter is a single-quote character, use
-             double-quote to prevent shell quoting problems. */
-          const char qc = (in_tab=='\'')?'"':'\'';
-          snprintf (tmp,sizeof (tmp),"-t %c%c%c ",qc,in_tab,qc);
-          strcat (cmd,tmp);
+#ifdef HAVE_ZERO_SORT
+          /* sort needs to understand -z to properly sort input using
+             NUL as the line delimiter */
+          args[argc++] = "-z";
+#else
+          die (EXIT_FAILURE, 0,
+                 _("-s and -z cannot be combined on this system"));
+#endif
         }
-      for (size_t i = 0; i < dm->num_grps; ++i)
+      if (!case_sensitive)
         {
-          const size_t col_num = dm->grps[i].num;
-          snprintf (tmp,sizeof (tmp),"-k%"PRIuMAX",%"PRIuMAX" ",
-                    (uintmax_t)col_num,(uintmax_t)col_num);
-          if (strlen (tmp)+strlen (cmd)+1 >= sizeof (cmd))
-            die (EXIT_FAILURE, 0,
-                   _("sort command too-long (please report this bug)"));
-          strcat (cmd,tmp);
+          args[argc++] = "-f";
         }
 
+      if (in_tab != TAB_WHITESPACE)
+        {
+          args[argc++] = "-t";
+          delim[0] = in_tab;
+          args[argc++] = delim;
+        }
+
+      int sort_spec = argc;
+      for (size_t i = 0; i < dm->num_grps; ++i)
+        {
+          char tmp[INT_BUFSIZE_BOUND (size_t) * 2 + 5];
+          const size_t col_num = dm->grps[i].num;
+          snprintf (tmp,sizeof (tmp),"-k%"PRIuMAX",%"PRIuMAX,
+                    (uintmax_t)col_num,(uintmax_t)col_num);
+          args[argc++] = xstrdup (tmp);
+        }
+
+      char *cmd = shell_quote_argv ((const char * const *) args);
+      while (sort_spec < argc)
+        {
+          free (args[sort_spec++]);
+        }
+      free (args);
       input_stream = popen (cmd,"r");
+      free (cmd);
       if (input_stream == NULL)
         die (EXIT_FAILURE, 0, _("failed to run 'sort': popen failed"));
     }
@@ -1138,6 +1214,8 @@ int main (int argc, char* argv[])
   DECL_LONG_DOUBLE_ROUNDING
   BEGIN_LONG_DOUBLE_ROUNDING ();
 
+  openbsd_pledge ();
+
   set_program_name (argv[0]);
 
 #ifdef FORCE_C_LOCALE
@@ -1159,9 +1237,9 @@ int main (int argc, char* argv[])
     {
       switch (optc)
         {
-	case 'C':
-	  skip_comments = true;
-	  break;
+        case 'C':
+          skip_comments = true;
+          break;
 
         case 'F':
           missing_field_filler = optarg;
@@ -1196,9 +1274,9 @@ int main (int argc, char* argv[])
           input_header = output_header = true;
           break;
 
-	case 'R':
-	  set_numeric_output_precision (optarg);
-	  break;
+        case 'R':
+          set_numeric_output_precision (optarg);
+          break;
 
         case 's':
           pipe_through_sort = true;
@@ -1223,18 +1301,29 @@ int main (int argc, char* argv[])
           in_tab = out_tab = optarg[0];
           break;
 
-	case OUTPUT_DELIMITER_OPTION:
+        case OUTPUT_DELIMITER_OPTION:
           if (optarg[0] == '\0' || optarg[1] != '\0')
             die (EXIT_FAILURE, 0,
-                   _("the delimiter must be a single character"));
-	  explicit_output_delimiter = (char)optarg[0];
-	  break;
+                 _("the delimiter must be a single character"));
+          explicit_output_delimiter = (char)optarg[0];
+          break;
 
         case 'W':
           in_tab = TAB_WHITESPACE;
           out_tab = '\t';
           break;
 
+        case SORT_PROGRAM_OPTION:
+          sort_cmd = xstrdup (optarg);
+          break;
+
+        case'c':
+          if (optarg[0] == '\0' || optarg[1] != '\0')
+            die (EXIT_FAILURE, 0,
+                 _("the delimiter must be a single character"));
+          collapse_separator = optarg[0];
+          break;
+
         case UNDOC_PRINT_INF_OPTION:
         case UNDOC_PRINT_NAN_OPTION:
           field_op_print_empty_value ( (optc==UNDOC_PRINT_INF_OPTION)
