@@ -1,5 +1,5 @@
 /* decorate -- auxiliary program for sort preprocessing
-   Copyright (C) 2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2020-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #define Version VERSION
 #include "version-etc.h"
 #include "ignore-value.h"
+#include "sh-quote.h"
 
 #include "decorate-functions.h"
 
@@ -38,7 +39,9 @@
 #define PROGRAM_NAME "decorate"
 
 #define AUTHORS \
-  proper_name ("Assaf Gordon")
+  proper_name ("Assaf Gordon, Shawn Wagner and Erik Auerswald")
+
+static const char *sort_cmd = SORT_PATH;
 
 static bool debug_print_pid = false;
 
@@ -89,7 +92,8 @@ enum
   NMERGE_OPTION,
   RANDOM_SOURCE_OPTION,
   HEADER_OPTION,
-  PARALLEL_OPTION
+  PARALLEL_OPTION,
+  SORT_PROGRAM_OPTION
 };
 
 static struct option const longopts[] =
@@ -101,6 +105,7 @@ static struct option const longopts[] =
   {"zero-terminated", no_argument, NULL, 'z'},
   {"print-sort-args", no_argument, NULL, PRINT_SORT_ARGS_OPTION},
   {"header", required_argument, NULL, HEADER_OPTION},
+  {"sort-cmd", required_argument, NULL, SORT_PROGRAM_OPTION},
 
   /* sort options, passed as-is to the sort process */
   {"check", optional_argument, NULL, CHECK_OPTION},
@@ -171,6 +176,12 @@ runs sort(1) and undecorates the result; This is the easiest method to use.\n\
 
      putchar ('\n');
 
+     fputs (_("\
+Options:\n\
+"), stdout);
+
+     putchar ('\n');
+
      fputs (_("\
 General Options:\n\
 "), stdout);
@@ -204,6 +215,9 @@ General Options:\n\
 "), stdout);
       fputs (_("\
   -z, --zero-terminated      line delimiter is NUL, not newline\n\
+"), stdout);
+      fputs (_("\
+      --sort-cmd=/path/to/sort   Alternative sort(1) to use.\n\
 "), stdout);
      fputs (HELP_OPTION_DESCRIPTION, stdout);
      fputs (VERSION_OPTION_DESCRIPTION, stdout);
@@ -211,7 +225,8 @@ General Options:\n\
      putchar ('\n');
 
      fputs (_("\
-The following options are passed to sort(1) as-is:\n\
+The following options are passed to sort as-is\
+ (Most of them assume GNU sort):\n\
 "), stdout);
 
      putchar ('\n');
@@ -285,7 +300,7 @@ Decorated output of roman numerals:\n\
 
 
      fputs (_("For detailed usage information and examples, see\n"),stdout);
-     printf ("  man %s\n", program_name);
+     printf ("  man %s\n", PROGRAM_NAME);
      fputs (_("The manual and more examples are available at\n"), stdout);
      fputs ("  " PACKAGE_URL "\n\n", stdout);
     }
@@ -442,7 +457,8 @@ decorate_file (const char *infile)
         }
 
       if (!decorate_fields (&lb))
-        die (SORT_FAILURE, errno, _("conversion failed in line %zu"), linenum);
+        die (SORT_FAILURE, errno, _("conversion failed in line %"PRIdMAX),
+             linenum);
 
       if (debug)
         dbg_printf ("writing line: '%.*s'", (int)lb.length, lb.buffer);
@@ -740,7 +756,8 @@ adjust_key_fields ()
 char**
 build_sort_process_args ()
 {
-  int argc = 2 ; /* one 'sort' program name (argv[0]), one for NULL */
+  int argc = 3 ; /* one 'sort' program name (argv[0]), one extra arg on NetBSD,
+                    one for NULL */
   struct keyfield *key = keylist;
 
   /* step 1: count number of args */
@@ -755,7 +772,17 @@ build_sort_process_args ()
   char** argv = xcalloc (argc, sizeof (char*));
   int i = 0;
 
-  argv[i++] = xstrdup ("sort"); /* argv[0] */
+  argv[i++] = xstrdup (sort_cmd); /* argv[0] */
+
+#ifdef __NetBSD__
+  /* NetBSD's /usr/bin/sort is stable by default, unlike every other
+   * one I checked, causing test case failures. So make it unstable.
+   */
+  if (STREQ (sort_cmd, "/usr/bin/sort"))
+    {
+      argv[i++] = xstrdup ("-S");
+    }
+#endif
 
   key = keylist;
   do {
@@ -770,6 +797,14 @@ build_sort_process_args ()
   return argv;
 }
 
+void
+free_sort_args (char **args)
+{
+  for (char **arg = args; *arg; ++arg) {
+    free (*arg);
+  }
+  free (args);
+}
 
 static void
 do_decorate (int optind, int argc, char** argv)
@@ -806,6 +841,8 @@ main (int argc, char **argv)
   bool decorate_only = false;
   bool print_sort_args = false;
 
+  openbsd_pledge ();
+
   set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -884,6 +921,10 @@ main (int argc, char **argv)
           add_sort_extra_args (optarg);
           break;
 
+        case SORT_PROGRAM_OPTION:
+          sort_cmd = xstrdup (optarg);
+          break;
+
         case RANDOM_SOURCE_OPTION:
           add_sort_extra_args ("--random-source");
           add_sort_extra_args (optarg);
@@ -991,16 +1032,11 @@ main (int argc, char **argv)
     {
       /* print and exit */
       char** sort_args = build_sort_process_args ();
-      char **p = sort_args;
-      while (*p)
-        {
-          /* TODO: shell quoting / escaping */
-          fputs (*p, stdout);
-          ++p;
-          if (*p)
-            fputc (' ', stdout);
-        }
-      fputc ('\n', stdout);
+      char *cmdline = shell_quote_argv ((const char * const*)sort_args);
+      puts (cmdline);
+      /* Not freeing these causes false positives with leak detectors */
+      free (cmdline);
+      free_sort_args (sort_args);
       exit (EXIT_SUCCESS);
     }
 
@@ -1121,7 +1157,7 @@ main (int argc, char **argv)
                   ++p;
                 }
             }
-          execvp ("sort", sort_args);
+          execvp (sort_cmd, sort_args);
 
           die (SORT_FAILURE, errno, _("failed to run the sort command"));
         }
