@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -29,6 +29,7 @@
 #include <stdint.h>
 #include <inttypes.h>
 #include <string.h>
+#include <strings.h>
 #include <stdbool.h>
 #include <time.h>
 #include <libgen.h> /* for dirname & POSIX version of basename */
@@ -80,6 +81,10 @@ struct operation_data operations[] =
   {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_HARMMEAN */
   {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_MS */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
+  /* OP_RMS */
+  {NUMERIC_SCALAR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_MEDIAN */
   {NUMERIC_VECTOR, IGNORE_FIRST, NUMERIC_RESULT},
   /* OP_QUARTILE_1 */
@@ -281,7 +286,7 @@ field_op_get_string_ptrs ( struct fieldop *op, bool sort,
   char *p = op->str_buf;
   const char* pend = op->str_buf + op->str_buf_used;
   size_t idx=0;
-  while (p < pend)
+  while (p < pend && idx < op->count)
     {
       ptrs[idx++] = p;
       while ( p<pend && *p != '\0' )
@@ -411,6 +416,11 @@ field_op_collect (struct fieldop *op,
       op->value += 1.0 / num_value;
       break;
 
+    case OP_MS:
+    case OP_RMS:
+      op->value += num_value * num_value;
+      break;
+
     case OP_COUNT:
       op->value++;
       break;
@@ -556,10 +566,16 @@ field_op_collect (struct fieldop *op,
     case OP_BIN_BUCKETS:
       {
         const long double val = num_value / op->params.bin_bucket_size;
-        modfl (val, & op->value);
-        /* signbit will take care of negative-zero as well. */
+        const long double frac = modfl (val, & op->value);
+        /* Buckets should follow this pattern:
+           ..., [-3x,-2x), [-2x,-x), [-x,0), [0,x), [x,2x), [2x,3x), ... */
         if (signbit (op->value))
-          --op->value;
+          {
+            if (is_zero (frac))
+                op->value = pos_zero (op->value);
+            else
+                --op->value;
+          }
         op->value *= op->params.bin_bucket_size;
       }
       break;
@@ -709,6 +725,8 @@ field_op_summarize_empty (struct fieldop *op)
     case OP_MEAN:
     case OP_GEOMEAN:
     case OP_HARMMEAN:
+    case OP_MS:
+    case OP_RMS:
     case OP_S_SKEWNESS:
     case OP_P_SKEWNESS:
     case OP_S_EXCESS_KURTOSIS:
@@ -820,6 +838,7 @@ field_op_summarize (struct fieldop *op)
   switch (op->op)                                /* LCOV_EXCL_BR_LINE */
     {
     case OP_MEAN:
+    case OP_MS:
       numeric_result = op->value / op->count;
       break;
 
@@ -892,7 +911,11 @@ field_op_summarize (struct fieldop *op)
     case OP_TRIMMED_MEAN:
       field_op_sort_values (op);
       numeric_result = trimmed_mean_value ( op->values, op->num_values,
-					    op->params.trimmed_mean);
+                                            op->params.trimmed_mean);
+      break;
+
+    case OP_RMS:
+      numeric_result = sqrtl (op->value / op->count);
       break;
 
     case OP_PSTDEV:
