@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -205,7 +205,7 @@ set_op_params (struct fieldop *op)
         op->params.trimmed_mean = _params[0].f;
       if (op->params.trimmed_mean<0 || op->params.trimmed_mean>0.5)
         die (EXIT_FAILURE, 0, _("invalid trim mean value %Lg " \
-				"(expected 0 <= X <= 0.5)"),
+             "(expected 0 <= X <= 0.5)"),
              op->params.trimmed_mean);
       if (_params_used>1)
         die (EXIT_FAILURE, 0, _("too many parameters for operation %s"),
@@ -524,7 +524,7 @@ parse_operations (enum processing_mode pm)
       tok = scanner_peek_token ();
     }
 
-  /* After adding all operations, see of there are master/slave ops
+  /* After adding all operations, see if there are master/slave ops
    * that need resolving - caching their pointer instead of index */
   for (size_t i=0; i<dm->num_ops; ++i)
      {
@@ -697,7 +697,13 @@ parse_mode ()
     if (dm->num_ops==0)
       {
         const uintmax_t grp_col = dm->grps[0].num;
-        struct parser_field_t dummy = {grp_col,false,NULL,false,false};
+        struct parser_field_t dummy = {
+          grp_col,              /* uintmax_t num;      */
+          dm->grps[0].by_name,  /* bool      by_name;  */
+          dm->grps[0].name,     /* char*     name;     */
+          false,                /* bool      range;    */
+          false                 /* bool      pair;     */
+        };
         add_op (OP_COUNT, &dummy);
       }
     else if (dm->num_ops>1)
@@ -750,7 +756,7 @@ parse_mode_or_op ()
     }
 
   die (EXIT_FAILURE,0, _("invalid operation %s"),
-		  quote (scanner_identifier));
+       quote (scanner_identifier));
 }
 
 struct datamash_ops*
@@ -798,6 +804,9 @@ datamash_ops_free ( struct datamash_ops* p )
   free (p->ops);
   p->ops = NULL;
 
+  for  (size_t i=0; i<_fields_used; ++i) {
+    free (_fields[i].name);
+  }
   free (_fields);
   _fields = NULL;
   _fields_alloc = 0;
