@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -43,13 +43,12 @@ int out_tab= '\t';
 bool case_sensitive = true;
 
 /* In the future: allow users to change this */
-char* numeric_output_format = "%.14Lg";
+char numeric_output_format[MAX_NUMERIC_FORMAT_LEN + 1] = "%.14Lg";
 
 /* number of bytes to allocate for output buffer */
 int   numeric_output_bufsize = 200;
 
 /* The character used to separate collapsed/uniqued strings */
-/* In the future: allow users to change this */
 char collapse_separator = ',';
 
 /* Should NA/NaN/empty values be silengtly ignored? */
@@ -105,7 +104,6 @@ set_numeric_output_precision (const char* digits)
 {
   long int l;
   char *p;
-  char tmp[100];
 
   if (digits == NULL || digits[0] == '\0')
     die (EXIT_FAILURE, 0, _("missing rounding digits value"));
@@ -114,10 +112,10 @@ set_numeric_output_precision (const char* digits)
   l = strtol (digits, &p, 10);
   if (errno != 0 || *p != '\0' || l <=0 || l> 50)
     die (EXIT_FAILURE, 0, _("invalid rounding digits value %s"),
-	 quote (digits));
+         quote (digits));
 
-  snprintf (tmp, sizeof (tmp), "%%.%dLf", (int)l);
-  numeric_output_format = xstrdup (tmp);
+  snprintf (numeric_output_format, sizeof (numeric_output_format), "%%.%dLf",
+            (int)l);
 
   finalize_numeric_output_buffer ();
 }
@@ -125,6 +123,9 @@ set_numeric_output_precision (const char* digits)
 void
 set_numeric_printf_format (const char* format)
 {
-  numeric_output_format = validate_double_format (format);
+  char *new_format = validate_double_format (format);
+  snprintf (numeric_output_format, sizeof (numeric_output_format), "%s",
+            new_format);
+  free (new_format);
   finalize_numeric_output_buffer ();
 }
