@@ -4,7 +4,7 @@
   Tests for 'transpose' and 'reverse' operation modes.
 
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -141,6 +141,31 @@ a	1	2
 b	3	XX
 EOF
 
+my $in4=<<'EOF';
+x y
+1 0.5
+2 1
+3 1.5
+4 2
+EOF
+
+my $out4_hdr=<<'EOF';
+GroupBy(x)	GroupBy(y)	count(x)
+	0.5	1	1.5	2
+1	1	N/A	N/A	N/A
+2	N/A	1	N/A	N/A
+3	N/A	N/A	1	N/A
+4	N/A	N/A	N/A	1
+EOF
+
+my $out4_no_hdr=<<'EOF';
+	0.5	1	1.5	2
+1	1	N/A	N/A	N/A
+2	N/A	1	N/A	N/A
+3	N/A	N/A	1	N/A
+4	N/A	N/A	N/A	1
+EOF
+
 my @Tests =
 (
   ['c1','crosstab 1,2 first 3', {IN_PIPE=>$in1}, {OUT=>$out1_first}],
@@ -163,28 +188,34 @@ my @Tests =
   ['c15','   ct 1,2 count 3',    {IN_PIPE=>$in2}, {OUT=>$out2_count_unsorted}],
   ['c16','-s ct 1,2 count 3',    {IN_PIPE=>$in2}, {OUT=>$out2_count_sorted}],
 
+  # test headers
+  ['c17','-W --header-in --header-out ct x,y',
+    {IN_PIPE=>$in4}, {OUT=>$out4_hdr}],
+  ['c18','-W --header-in ct x,y',
+    {IN_PIPE=>$in4}, {OUT=>$out4_no_hdr}],
+
   # Test missing values
   ['c30','ct 1,2 first 3',       {IN_PIPE=>$in3}, {OUT=>$out3_na}],
   ['c31','--filler XX ct 1,2 first 3',       {IN_PIPE=>$in3}, {OUT=>$out3_xx}],
 
   # Test wrong usage
   ['e1',  'ct',  {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: missing field for operation 'crosstab'\n"}],
+    {ERR=>"$prog: missing field for operation 'crosstab'\n"}],
   ['e2',  'ct 1',  {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: crosstab requires exactly 2 fields, found 1\n"}],
+    {ERR=>"$prog: crosstab requires exactly 2 fields, found 1\n"}],
   ['e3',  'ct 1,2,3,4',  {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: crosstab requires exactly 2 fields, found 4\n"}],
+    {ERR=>"$prog: crosstab requires exactly 2 fields, found 4\n"}],
   ['e4',  'ct 1,2 md5 4', {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: conflicting operation found: expecting crosstab " .
-            "operations, but found line operation 'md5'\n"}],
+    {ERR=>"$prog: conflicting operation found: expecting crosstab " .
+          "operations, but found line operation 'md5'\n"}],
   ['e5',  'ct 1,2 sum 1,2', {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: crosstab supports one operation, found 2\n"}],
+    {ERR=>"$prog: crosstab supports one operation, found 2\n"}],
   ['e6',  'ct 1,2 min 2 max 2', {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: crosstab supports one operation, found 2\n"}],
+    {ERR=>"$prog: crosstab supports one operation, found 2\n"}],
   ['e7',  'ct 1:2', {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: invalid field pair for operation 'crosstab'\n"}],
+    {ERR=>"$prog: invalid field pair for operation 'crosstab'\n"}],
   ['e8',  'ct 1-2', {IN_PIPE=>""}, {EXIT=>1},
-      {ERR=>"$prog: invalid field range for operation 'crosstab'\n"}],
+    {ERR=>"$prog: invalid field range for operation 'crosstab'\n"}],
 );
 
 my $save_temps = $ENV{SAVE_TEMPS};
