@@ -2,7 +2,7 @@
 =pod
   Unit Tests for GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com
 
    This file is part of GNU Datamash.
 
@@ -187,6 +187,14 @@ The datamash tests below should return the same results are thes R commands:
     trimmean40=function(x){mean(x,trim=0.4)}
     trimmean50=function(x){mean(x,trim=0.5)}
 
+    # Package for harmonic and geometric mean
+    library(psych)
+
+    # Helper function for ms
+    ms=function(x) { mean(x^2) }
+
+    # Helper function for rms
+    rms=function(x) { sqrt(mean(x^2)) }
 
     # Helper function for madraw
     madraw=function(x) { mad(x,constant=1.0) }
@@ -245,6 +253,8 @@ The datamash tests below should return the same results are thes R commands:
     # Run tests
     test(range)
     test(mean)
+    test(geometric.mean)
+    test(harmonic.mean)
     test(median)
     test(q1)
     test(q3)
@@ -257,7 +267,7 @@ The datamash tests below should return the same results are thes R commands:
     test(trimmean30)
     test(trimmean40)
     test(trimmean50)
-    test(iqr)
+    test(IQR)
     test(smp.sd)
     test(pop.sd)
     test(smp.var)
@@ -269,6 +279,8 @@ The datamash tests below should return the same results are thes R commands:
     test(pop.excess_kurtosis)
     test(smp.excess_kurtosis)
     test(jarque.bera.pvalue)
+    test(ms)
+    test(rms)
 
 =cut
 
@@ -291,7 +303,7 @@ my @Tests =
   ['mean7', 'mean 1' ,  {IN_PIPE=>$seq11}, {OUT => "14.545\n"},],
   ['mean8', 'mean 1' ,  {IN_PIPE=>$seq12_unsorted}, {OUT => "16.416\n"},],
   ['mean9', '--sort mean 1' ,
-     {IN_PIPE=>$seq12_unsorted}, {OUT => "16.416\n"},],
+    {IN_PIPE=>$seq12_unsorted}, {OUT => "16.416\n"},],
   ['mean10','mean 1' ,  {IN_PIPE=>$seq20}, {OUT => "100.06\n"},],
   ['mean11','mean 1' ,  {IN_PIPE=>$seq21}, {OUT => "45.32\n"},],
   ['mean12','mean 1' ,  {IN_PIPE=>$seq22}, {OUT => "67.45\n"},],
@@ -308,7 +320,7 @@ my @Tests =
   ['geomean7', 'geomean 1' ,  {IN_PIPE=>$seq11}, {OUT => "10.653\n"},],
   ['geomean8', 'geomean 1' ,  {IN_PIPE=>$seq12_unsorted}, {OUT => "11.817\n"},],
   ['geomean9', '--sort geomean 1' ,
-     {IN_PIPE=>$seq12_unsorted}, {OUT => "11.817\n"},],
+    {IN_PIPE=>$seq12_unsorted}, {OUT => "11.817\n"},],
   ['geomean10','geomean 1' ,  {IN_PIPE=>$seq20}, {OUT => "99.596\n"},],
   ['geomean11','geomean 1' ,  {IN_PIPE=>$seq21}, {OUT => "34.644\n"},],
   ['geomean12','geomean 1' ,  {IN_PIPE=>$seq22}, {OUT => "67.386\n"},],
@@ -323,13 +335,39 @@ my @Tests =
   ['harmmean6', 'harmmean 1' ,  {IN_PIPE=>$seq12}, {OUT => "7.534\n"},],
   ['harmmean7', 'harmmean 1' ,  {IN_PIPE=>$seq11}, {OUT => "7.025\n"},],
   ['harmmean8', 'harmmean 1' ,  {IN_PIPE=>$seq12_unsorted},
-     {OUT => "7.534\n"},],
+    {OUT => "7.534\n"},],
   ['harmmean9', '--sort harmmean 1' ,
-     {IN_PIPE=>$seq12_unsorted}, {OUT => "7.534\n"},],
+    {IN_PIPE=>$seq12_unsorted}, {OUT => "7.534\n"},],
   ['harmmean10','harmmean 1' ,  {IN_PIPE=>$seq20}, {OUT => "99.122\n"},],
   ['harmmean11','harmmean 1' ,  {IN_PIPE=>$seq21}, {OUT => "19.563\n"},],
   ['harmmean12','harmmean 1' ,  {IN_PIPE=>$seq22}, {OUT => "67.322\n"},],
 
+  # Test mean square
+  ['ms1', 'ms 1', {IN_PIPE=>$seq1}, {OUT => "7.5\n"},],
+  ['ms2', 'ms 1', {IN_PIPE=>$seq2}, {OUT => "4.666\n"},],
+  ['ms3', 'ms 1', {IN_PIPE=>$seq3}, {OUT => "4\n"},],
+  ['ms4', 'ms 1', {IN_PIPE=>$seq9}, {OUT => "172.888\n"},],
+  ['ms5', 'ms 1', {IN_PIPE=>$seq10}, {OUT => "239.7\n"},],
+  ['ms6', 'ms 1', {IN_PIPE=>$seq11}, {OUT => "305.272\n"},],
+  ['ms7', 'ms 1', {IN_PIPE=>$seq12}, {OUT => "393.916\n"},],
+  ['ms8', 'ms 1', {IN_PIPE=>$seq20}, {OUT => "10102.8\n"},],
+  ['ms9', 'ms 1', {IN_PIPE=>$seq21}, {OUT => "2971.76\n"},],
+  ['ms10', 'ms 1', {IN_PIPE=>$seq22}, {OUT => "4558.03\n"},],
+  ['ms11', 'ms 1', {IN_PIPE=>$seq23}, {OUT => "42.689\n"},],
+
+  # Test root mean square
+  ['rms1', 'rms 1', {IN_PIPE=>$seq1}, {OUT => "2.738\n"},],
+  ['rms2', 'rms 1', {IN_PIPE=>$seq2}, {OUT => "2.160\n"},],
+  ['rms3', 'rms 1', {IN_PIPE=>$seq3}, {OUT => "2\n"},],
+  ['rms4', 'rms 1', {IN_PIPE=>$seq9}, {OUT => "13.148\n"},],
+  ['rms5', 'rms 1', {IN_PIPE=>$seq10}, {OUT => "15.482\n"},],
+  ['rms6', 'rms 1', {IN_PIPE=>$seq11}, {OUT => "17.472\n"},],
+  ['rms7', 'rms 1', {IN_PIPE=>$seq12}, {OUT => "19.847\n"},],
+  ['rms8', 'rms 1', {IN_PIPE=>$seq20}, {OUT => "100.512\n"},],
+  ['rms9', 'rms 1', {IN_PIPE=>$seq21}, {OUT => "54.513\n"},],
+  ['rms10', 'rms 1', {IN_PIPE=>$seq22}, {OUT => "67.513\n"},],
+  ['rms11', 'rms 1', {IN_PIPE=>$seq23}, {OUT => "6.533\n"},],
+
   # Test median
   ['med1', 'median 1' ,  {IN_PIPE=>$seq1},  {OUT => "2.5\n"}],
   ['med2', 'median 1' ,  {IN_PIPE=>$seq2},  {OUT => "2\n"}],
@@ -609,9 +647,9 @@ my @Tests =
   ['mad_6', 'mad 1' ,  {IN_PIPE=>$seq11},  {OUT => "11.860\n"}],
   ['mad_7', 'mad 1' ,  {IN_PIPE=>$seq12},  {OUT => "13.343\n"}],
   ['mad_7.1', 'mad 1' ,  {IN_PIPE=>$seq12_unsorted},
-      {OUT => "13.343\n"}],
+    {OUT => "13.343\n"}],
   ['mad_7.2', '--sort mad 1' ,  {IN_PIPE=>$seq12_unsorted},
-     {OUT => "13.343\n"}],
+    {OUT => "13.343\n"}],
   ['mad_8', 'mad 1' ,  {IN_PIPE=>$seq20},  {OUT => "10.378\n"},],
   ['mad_9', 'mad 1' ,  {IN_PIPE=>$seq21},  {OUT => "27.428\n"},],
   ['mad_10','mad 1' ,  {IN_PIPE=>$seq22},  {OUT => "4.447\n"},],
@@ -626,9 +664,9 @@ my @Tests =
   ['madraw_6', 'madraw 1' ,  {IN_PIPE=>$seq11},  {OUT => "8\n"}],
   ['madraw_7', 'madraw 1' ,  {IN_PIPE=>$seq12},  {OUT => "9\n"}],
   ['madraw_7.1', 'madraw 1' ,  {IN_PIPE=>$seq12_unsorted},
-      {OUT => "9\n"}],
+    {OUT => "9\n"}],
   ['madraw_7.2', '--sort madraw 1' ,  {IN_PIPE=>$seq12_unsorted},
-      {OUT => "9\n"}],
+    {OUT => "9\n"}],
   ['madraw_8', 'madraw 1' ,  {IN_PIPE=>$seq20},  {OUT => "7\n"},],
   ['madraw_9', 'madraw 1' ,  {IN_PIPE=>$seq21},  {OUT => "18.5\n"},],
   ['madraw_10','madraw 1' ,  {IN_PIPE=>$seq22},  {OUT => "3\n"},],
@@ -712,7 +750,6 @@ my @Tests =
   ['dpo_9', 'dpo 1' ,  {IN_PIPE=>$seq21},  {OUT => "7.689e-10\n"},],
   ['dpo_10','dpo 1' ,  {IN_PIPE=>$seq22},  {OUT => "0.819\n"},],
   ['dpo_11','dpo 1' ,  {IN_PIPE=>$seq23},  {OUT => "0.002\n"},],
-
 );
 
 ##
