@@ -1,7 +1,7 @@
 #!/bin/sh
 #   Unit Tests for GNU Datamash - perform simple calculation on input data
 
-#    Copyright (C) 2015-2020 Assaf Gordon <assafgordon@gmail.com>
+#    Copyright (C) 2015-2021 Assaf Gordon <assafgordon@gmail.com>
 #
 #    This file is part of GNU Datamash.
 #
@@ -28,7 +28,8 @@
 
 fail=0
 
-## Ensure seq,awk are useable
+## Ensure seq is useable
+openbsd_seq_replacement_
 seq 10 >/dev/null 2>/dev/null \
     || skip_ "requires a working seq"
 
