@@ -2,7 +2,7 @@
 =pod
   Unit Tests for GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -56,16 +56,6 @@ die "test infrastructure failed: can't determine 'nan' string" unless $nan;
 my $inf = `$prog_bin ---print-inf`;
 die "test infrastructure failed: can't determine 'inf' string" unless $inf;
 
-## Portability hack
-## Check if the system's sort supports stable sorting ('-s').
-## If it doesn't - skip some tests
-my $rc = system("sort -s < /dev/null > /dev/null 2>/dev/null");
-die "testing framework failure: failed to execute sort -s"
-  if ( ($rc == -1) || ($rc & 127) );
-my $sort_exit_code = ($rc >> 8);
-my $have_stable_sort = ($sort_exit_code==0);
-
-
 # TODO: add localization tests with "grouping"
 # Turn off localization of executable's output.
 @ENV{qw(LANGUAGE LANG LC_ALL)} = ('C') x 3;
@@ -215,24 +205,36 @@ my $bin_out1_5_5=<<'EOF';
 EOF
 
 my $bin_in2=<<'EOF';
+-101.9
+-3.1
 -3
 -0.3
+-0
+0
 0.3
 3
 103
 EOF
 
 my $bin_out2_100=<<'EOF';
+-200
 -100
 -100
+-100
+0
+0
 0
 0
 100
 EOF
 
 my $bin_out2_3=<<'EOF';
+-102
 -6
 -3
+-3
+0
+0
 0
 3
 102
@@ -371,44 +373,24 @@ a,b,c
 1,X,6
 EOF
 
-
-
 my @Tests =
 (
   # Test 'min' + --full
   # first, verify test without "--full"
   ['slct1', '-t" " -g1 min 2', {IN_PIPE=>$in_full1}, {OUT=>"A 3\nB 0\n"}],
-  # Test with "--full", "i2" and "i6" should be displayed
-  ['slct2', '-t" " -f -g1 min 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 3 i2 3\nB 0 i6 0\n"}],
- # --full with --sort => should not change results
-  ['slct3', '-s -t" " -f -g1 min 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 3 i2 3\nB 0 i6 0\n"}],
 
   # Test 'max' + --full
   # first, verify test without "--full"
   ['slct4', '-t" " -g1 max 2', {IN_PIPE=>$in_full1}, {OUT=>"A 5\nB 8\n"}],
-  # Test with "--full", "i3" and "i7" should be displayed
-  ['slct5', '-t" " -f -g1 max 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 5 i3 5\nB 8 i5 8\n"}],
-  # --full with --sort => should not change results
-  ['slct6', '-s -t" " -f -g1 max 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 5 i3 5\nB 8 i5 8\n"}],
 
   # Test 'first' + --full
   # first, verify test without "--full"
   ['slct7', '-t" " -g1 first 2', {IN_PIPE=>$in_full1}, {OUT=>"A 4\nB 1\n"}],
-  # Test with "--full", "i1" and "i4" should be displayed
-  ['slct8', '-t" " -f -g1 first 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 4 i1 4\nB 1 i4 1\n"}],
   # more --full with --sort => see test 'sortslct1' below
 
   # Test 'last' + --full
   # first, verify test without "--full"
   ['slct9', '-t" " -g1 last 2', {IN_PIPE=>$in_full1}, {OUT=>"A 5\nB 3\n"}],
-  # Test with "--full", "i1" and "i4" should be displayed
-  ['slct10', '-t" " -f -g1 last 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 5 i3 5\nB 3 i7 3\n"}],
   # more --full with --sort => see test 'sortslct2' below
 
 
@@ -419,42 +401,42 @@ my @Tests =
   ['narm2', '--narm mean 1', {IN_PIPE=>$na1}, {OUT=>"2\n"}],
   # without --narm, these should fail with invalid input
   ['narm3', 'sum 1',         {IN_PIPE=>$na1}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NA'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NA'\n"}],
 
   ## Test with 'nA'
   ['narm4', '--narm sum 1',  {IN_PIPE=>$na2}, {OUT=>"6\n"}],
   ['narm5', '--narm mean 1', {IN_PIPE=>$na2}, {OUT=>"2\n"}],
   # without --narm, these should fail with invalid input
   ['narm6', 'sum 1',         {IN_PIPE=>$na2}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 3 field 1: 'nA'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 3 field 1: 'nA'\n"}],
 
   ## Test with 'N/A'
   ['narm7', '--narm sum 1',  {IN_PIPE=>$na3}, {OUT=>"6\n"}],
   ['narm8', '--narm mean 1', {IN_PIPE=>$na3}, {OUT=>"2\n"}],
   # without --narm, these should fail with invalid input
   ['narm9', 'sum 1',         {IN_PIPE=>$na3}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 3 field 1: 'N/A'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 3 field 1: 'N/A'\n"}],
 
   ## Test with 'NaN'
   ['narm10', '--narm sum 1',  {IN_PIPE=>$na4}, {OUT=>"6\n"}],
   ['narm11', '--narm mean 1', {IN_PIPE=>$na4}, {OUT=>"2\n"}],
   # without --narm, 'nan' should be processed, not skipped
   ['narm12', 'sum 1',         {IN_PIPE=>$na4}, {OUT=>"$nan\n"},
-      {OUT_SUBST=>'s/^-//'}],
+    {OUT_SUBST=>'s/^-//'}],
 
   ## Test with 'nan'
   ['narm13', '--narm sum 1',  {IN_PIPE=>$na5}, {OUT=>"6\n"}],
   ['narm14', '--narm mean 1', {IN_PIPE=>$na5}, {OUT=>"2\n"}],
   # without --narm, 'nan' should be processed, not skipped
   ['narm15', 'sum 1',         {IN_PIPE=>$na5}, {OUT=>"$nan\n"},
-      {OUT_SUBST=>'s/^-//'}],
+    {OUT_SUBST=>'s/^-//'}],
 
   # These input have strings starting with 'NA' or 'NAN' but should not
   # be mistaken for them.
   ['narm16', '--narm sum 1',         {IN_PIPE=>$not_na1}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NAK'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NAK'\n"}],
   ['narm17', '--narm sum 1',         {IN_PIPE=>$not_na2}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NANA'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 3 field 1: 'NANA'\n"}],
 
   ## NA value as first/last line
   ['narm18', '--narm mean 1',  {IN_PIPE=>$na_first}, {OUT=>"5\n"}],
@@ -464,13 +446,13 @@ my @Tests =
   ['narm20', '-t: --narm sum 1 sum 2 sum 3', {IN_PIPE=>$na_mid1},
     {OUT=>"12:10:18\n"}],
   ['narm21', '-t:        sum 1 sum 2 sum 3', {IN_PIPE=>$na_mid1}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 2 field 2: 'NA'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 2 field 2: 'NA'\n"}],
 
   ## NA as the last field
   ['narm22', '-t: --narm sum 1 sum 2 sum 3', {IN_PIPE=>$na_mid1},
     {OUT=>"12:10:18\n"}],
   ['narm23', '-t:        sum 1 sum 2 sum 3', {IN_PIPE=>$na_mid1}, {EXIT=>1},
-      {ERR=>"$prog: invalid numeric value in line 2 field 2: 'NA'\n"}],
+    {ERR=>"$prog: invalid numeric value in line 2 field 2: 'NA'\n"}],
 
   # N/A affect counts
   ['narm24', '-t: count 1 count 2 count 3', {IN_PIPE=>$na_last1},
@@ -541,11 +523,16 @@ my @Tests =
 
 
   # Test binning
-  ['bin1', 'bin 1',     {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_100}],
-  ['bin2', 'bin:5 1',   {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_5}],
-  ['bin3', 'bin:5.5 1', {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_5_5}],
-  ['bin4', 'bin 1',     {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_100}],
-  ['bin5', 'bin:3 1',   {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin1', 'bin 1',        {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_100}],
+  ['bin2', 'bin:5 1',      {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_5}],
+  ['bin3', 'bin:5.5 1',    {IN_PIPE=>$bin_in1}, {OUT=>$bin_out1_5_5}],
+  ['bin4', 'bin 1',        {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_100}],
+  ['bin5', 'bin:3 1',      {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin6', 'bin:3. 1',     {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin7', 'bin:3.0 1',    {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin8', 'bin:3.e0 1',   {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin9', 'bin:0.3e1 1',  {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
+  ['bin10','bin:30.e-1 1', {IN_PIPE=>$bin_in2}, {OUT=>$bin_out2_3}],
 
   # Test rounding functions
   ['rnd1', '-H --full round 1 floor 1 ceil 1 trunc 1 frac 1',
@@ -563,35 +550,42 @@ my @Tests =
   # Test output-delimiter
   ['odlm1', '-t: last 1 last 2', {IN_PIPE=>$na_mid1}, {OUT=>"7:8\n"}],
   ['odlm2', '-t: --output-delimiter "%" last 1 last 2',
-   {IN_PIPE=>$na_mid1}, {OUT=>"7%8\n"}],
+    {IN_PIPE=>$na_mid1}, {OUT=>"7%8\n"}],
   ['odlm3', '--output-delimiter "%" -t: last 1 last 2',
-   {IN_PIPE=>$na_mid1}, {OUT=>"7%8\n"}],
+    {IN_PIPE=>$na_mid1}, {OUT=>"7%8\n"}],
 
   # output-delimiter with whitespace
   ['odlm4', '-t " " last 1 last 2', {IN_PIPE=>$in_full1}, {OUT=>"B 3\n"}],
   ['odlm5', '-W     last 1 last 2', {IN_PIPE=>$in_full1}, {OUT=>"B\t3\n"}],
   ['odlm6', '-W --output-delimiter ":" last 1 last 2',
-   {IN_PIPE=>$in_full1}, {OUT=>"B:3\n"}],
+    {IN_PIPE=>$in_full1}, {OUT=>"B:3\n"}],
   ['odlm7', '--output-delimiter ":" -W last 1 last 2',
-   {IN_PIPE=>$in_full1}, {OUT=>"B:3\n"}],
+    {IN_PIPE=>$in_full1}, {OUT=>"B:3\n"}],
 
   # Multiple output delimiters
   ['odlm8', '--output-delimiter "x" -W --output-delimiter "y" last 1 last 2',
-   {IN_PIPE=>$in_full1}, {OUT=>"By3\n"}],
+    {IN_PIPE=>$in_full1}, {OUT=>"By3\n"}],
 
 
   # Test -C/--skip-comments option
   ['sc1', 'sum 2',      {IN_PIPE=>$in_comments}, {OUT=>"15\n"}],
   ['sc2', '-C sum 2',   {IN_PIPE=>$in_comments}, {OUT=>"5\n"}],
   ['sc3', 'reverse',    {IN_PIPE=>$in_comments},
-   {OUT=>"3	 #foo\n" .
-	 "5	bar\n" .
-	 "7	;baz\n"}],
+    {OUT=>"3	 #foo\n" .
+          "5	bar\n" .
+          "7	;baz\n"}],
   ['sc4', '-C reverse', {IN_PIPE=>$in_comments}, {OUT=>"5	bar\n"}],
 
 
+  # Regression tests
+  # TODO: Move regression tests into a separate file when sufficiently many
+  #       have accumulated to warrant the overhead of another test file.
   # Bug in mode/antimode in 1.4 and earlier
   ['bug_mode1', 'mode 1', {IN_PIPE=>"-1"}, {OUT=>"-1\n"}],
+  # "Segmentation fault" when input contains embedded NUL characters
+  # (see https://lists.gnu.org/archive/html/bug-datamash/2020-11/msg00001.html)
+  ['regr001', 'countunique 1', {IN_PIPE=>"\x00" x 100}, {OUT=>"1\n"}],
+  ['regr002', 'unique 1',      {IN_PIPE=>"\x00" x 100}, {OUT=>"\n"}],
 
   ##
   ## Backslash escaping in identifiers
@@ -599,23 +593,22 @@ my @Tests =
 
   ## minus in field name
   ['esc1', '-W -H sum "A_Chlor_T1h_r1\\-metaG"', {IN_PIPE=>$in_esc_ident},
-     {OUT=>"sum(A_Chlor_T1h_r1-metaG)\n11\n"}],
+    {OUT=>"sum(A_Chlor_T1h_r1-metaG)\n11\n"}],
   # field name starting with an identifier
   ['esc2', '-W -H sum "\\9C"', {IN_PIPE=>$in_esc_ident},
-   {OUT=>"sum(9C)\n33\n"}],
+    {OUT=>"sum(9C)\n33\n"}],
   # field name starting with minus
   ['esc3', '-W -H sum "\\-bar"', {IN_PIPE=>$in_esc_ident},
-     {OUT=>"sum(-bar)\n44\n"}],
+    {OUT=>"sum(-bar)\n44\n"}],
 
 
   # dirname and basename
   ['dnbn1', 'dirname 1 basename 1', {IN_PIPE=>$in_dirname_basename},
-     {OUT=>$exp_dirname_basename}],
+    {OUT=>$exp_dirname_basename}],
 
   # barename and extname
   ['bnen1', 'barename 1 extname 1', {IN_PIPE=>$in_barename_extname},
-     {OUT=>$exp_barename_extname}],
-
+    {OUT=>$exp_barename_extname}],
 
 
   ## GetNum variants
@@ -640,41 +633,51 @@ my @Tests =
   ['gn12', 'getnum:i 1', {IN_PIPE=>"moo---4"}, {OUT=>"0\n"}],
   ['gn13', 'getnum:d 1', {IN_PIPE=>"moo...4"}, {OUT=>"0\n"}],
   ['gn14', 'getnum:i 1',
-   {IN_PIPE=>"moo999999999999999999999999999991234312341432123451541341312431"},
-   {OUT=>"0\n"}],
+    {IN_PIPE=>"moo99999999999999999999999999999" .
+              "1234312341432123451541341312431"},
+    {OUT=>"0\n"}],
   ['gn15', 'getnum:d 1',
-   {IN_PIPE=>"moo" . "9" x 10000},
-   {OUT=>"0\n"}],
+    {IN_PIPE=>"moo" . "9" x 10000},
+    {OUT=>"0\n"}],
   ['gn16', 'getnum:n 1', {IN_PIPE=>"moo" x 4000 . "42" . "bar" x 1000},
-   {OUT=>"42\n"}],
+    {OUT=>"42\n"}],
 
 
   # cut operation
   ['cut1','-t, cut 3,1,2', {IN_PIPE=>$in1_cut}, {OUT=>"c,a,b\n6,1,X\n"}],
   ['cut2','-t, -H cut c,a', {IN_PIPE=>$in1_cut}, {OUT=>"cut(c),cut(a)\n6,1\n"}],
 
+  # Aliases (more-or-less: copy-paste above tests and s/orig/alias/ the
+  # command names.)
+  ['alias1','-t, echo 3,1,2', {IN_PIPE=>$in1_cut}, {OUT=>"c,a,b\n6,1,X\n"}],
+  ['alias2','-t, -H echo c,a', {IN_PIPE=>$in1_cut},
+    {OUT=>"cut(c),cut(a)\n6,1\n"}],
+  ['alias3', '-t: --narm uniq 2', {IN_PIPE=>$na_mid1},
+    {OUT=>"2,8\n"}],
+  ['alias4', '-t: uniq 2', {IN_PIPE=>$na_mid1},
+    {OUT=>"2,8,NA\n"}],
+  ['alias5', '--narm uniq 1',  {IN_PIPE=>$na_all}, {OUT=>"\n"}],
+
+  # Decimal separator used as input field delimiter
+  ['ifdl01', '-t. cut 2,1',           {IN_PIPE=>"1.2\n"},   {OUT=>"2.1\n"}],
+  ['ifdl02', '-t. cut 3,2,1',         {IN_PIPE=>"1.2.3\n"}, {OUT=>"3.2.1\n"}],
+  ['ifdl03', '-t. unique 1,2',        {IN_PIPE=>"1.2\n"},   {OUT=>"1.2\n"}],
+  ['ifdl04', '-t. unique 1,2,3',      {IN_PIPE=>"1.2.3\n"}, {OUT=>"1.2.3\n"}],
+  ['ifdl05', '-t. count 1,2',         {IN_PIPE=>"1.2\n"},   {OUT=>"1.1\n"}],
+  ['ifdl06', '-t. count 1,2,3',       {IN_PIPE=>"1.2.3\n"}, {OUT=>"1.1.1\n"}],
+  ['ifdl07', '-t. countunique 1,2',   {IN_PIPE=>"1.2\n"},   {OUT=>"1.1\n"}],
+  ['ifdl08', '-t. countunique 1,2,3', {IN_PIPE=>"1.2.3\n"}, {OUT=>"1.1.1\n"}],
+  ['ifdl09', '-t. rmdup 1',           {IN_PIPE=>"1.2\n"},   {OUT=>"1.2\n"}],
+  ['ifdl10', '-t. rmdup 2',           {IN_PIPE=>"1.2\n"},   {OUT=>"1.2\n"}],
+  ['ifdl11', '-t. rmdup 3',           {IN_PIPE=>"1.2.3\n"}, {OUT=>"1.2.3\n"}],
+  # TODO: fix bug for numeric operations
+  #['ifdl12', '-t. sum 1,2',           {IN_PIPE=>"1.2\n"},   {OUT=>"1.2\n"}],
+  #['ifdl13', '-t. sum 1,2,3',         {IN_PIPE=>"1.2.3\n"}, {OUT=>"1.2.3\n"}],
+  # TODO: fix bug for "getnum" operation
+  #['ifdl14', '-t. getnum 1,2',        {IN_PIPE=>"x1.2y\n"}, {OUT=>"1.2\n"}],
+
 );
 
-
-if ($have_stable_sort) {
-  push @Tests, (
-  # Test 'first' + --full + --sort
-  # NOTE: This is subtle:
-  #       Sorting should be stable: only ordering the column which is used
-  #       for grouping (column 1 in this test). This means that the second
-  #       column (containing numbers) should NOT affect sorting, and the order
-  #       of the lines should not change. The results of this test
-  #       should be the same as 'slct8'. If the system doesn't have stable
-  #       'sort', then the order will change.
-  ['sortslct1', '-s -t" " -f -g1 first 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 4 i1 4\nB 1 i4 1\n"}],
-  # Test 'last' + --full + --sort
-  # See note above regarding 'first' - applies to 'last' as well.
-  ['sortslct2', '-s -t" " -f -g1 last 2', {IN_PIPE=>$in_full1},
-    {OUT=>"A 5 i3 5\nB 3 i7 3\n"}],
-  )
-}
-
 my $save_temps = $ENV{SAVE_TEMPS};
 my $verbose = $ENV{VERBOSE};
 
