@@ -2,7 +2,7 @@
 =pod
   Unit Tests for decorate
 
-   Copyright (C) 2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2020-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -33,7 +33,7 @@ use MIME::Base64 ;
 
 (my $program_name = $0) =~ s|.*/||;
 my $prog = 'decorate';
-
+$prog .= " --sort-cmd=/usr/bin/sort" if $^O eq "netbsd";
 
 # TODO: add localization tests with "grouping"
 # Turn off localization of executable's output.
@@ -177,32 +177,75 @@ D   192.168.17.8
 M   192.168.43.1
 EOF
 
+my $in2=<<'EOF';
+203.0.113.47
+192.0.2.33
+203.0.113.0
+192.0.2.3
+0.0.0.0
+::ffff:192.0.2.42
+2001:db8:6:5:4:3:2:1
+2001:Db8::
+::192.0.2.41
+::1
+EOF
+
+my $out2_ipv6v4map=<<'EOF';
+::1
+::192.0.2.41
+0.0.0.0
+192.0.2.3
+192.0.2.33
+::ffff:192.0.2.42
+203.0.113.0
+203.0.113.47
+2001:Db8::
+2001:db8:6:5:4:3:2:1
+EOF
+
+my $out2_ipv6v4comp=<<'EOF';
+0.0.0.0
+::1
+192.0.2.3
+192.0.2.33
+::192.0.2.41
+203.0.113.0
+203.0.113.47
+::ffff:192.0.2.42
+2001:Db8::
+2001:db8:6:5:4:3:2:1
+EOF
+
 
 my @Tests =
 (
- ['s1', '-k1,1:roman', {IN_PIPE=>$in1}, {OUT => $out1_dec_roman}],
- ['s2', '-k2,2:ipv4',  {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4}],
- ['s3', '-k2,2:ipv4 --stable', {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4_stable}],
- ['s4', '-k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4_rev}],
+  ['s1', '-k1,1:roman', {IN_PIPE=>$in1}, {OUT => $out1_dec_roman}],
+  ['s2', '-k2,2:ipv4',  {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4}],
+  ['s3', '-k2,2:ipv4 --stable', {IN_PIPE=>$in1},
+    {OUT => $out1_dec_ipv4_stable}],
+  ['s4', '-k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4_rev}],
 
- ['s5', '-k1,1:roman -k2,2r:ipv4' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_roman_ipv4rev}],
- ['s6', '-k2,2:ipv4 -k1r,1:roman' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_ipv4_romanrev}],
+  ['s5', '-k1,1:roman -k2,2r:ipv4' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_roman_ipv4rev}],
+  ['s6', '-k2,2:ipv4 -k1r,1:roman' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_ipv4_romanrev}],
 
- ['s10', '-k1,1:roman -k2,2' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_roman_k2}],
- ['s11', '-k2n,2 -k1,1:roman' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_k2n_roman}],
+  ['s10', '-k1,1:roman -k2,2' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_roman_k2}],
+  ['s11', '-k2n,2 -k1,1:roman' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_k2n_roman}],
+
+  ['s12', '-k1,1:ipv6v4map',  {IN_PIPE=>$in2}, {OUT => $out2_ipv6v4map}],
+  ['s13', '-k1,1:ipv6v4comp', {IN_PIPE=>$in2}, {OUT => $out2_ipv6v4comp}],
 
 
- # Sort with header lines
- ['sh1', '-H -k2,2r:ipv4', {IN_PIPE=>$in1}, {OUT=>$out1_dec_ipv4_rev_header1}],
- ['sh2', '--header=2 -k2,2r:ipv4',  {IN_PIPE=>$in1},
-  {OUT => $out1_dec_ipv4_rev_header2}],
- # More header lines than in the input
- ['sh3', '--header=9 -k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $in1}],
- ['sh4', '--header=10 -k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $in1}],
+  # Sort with header lines
+  ['sh1', '-H -k2,2r:ipv4', {IN_PIPE=>$in1}, {OUT=>$out1_dec_ipv4_rev_header1}],
+  ['sh2', '--header=2 -k2,2r:ipv4',  {IN_PIPE=>$in1},
+    {OUT => $out1_dec_ipv4_rev_header2}],
+  # More header lines than in the input
+  ['sh3', '--header=9 -k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $in1}],
+  ['sh4', '--header=10 -k2,2r:ipv4',  {IN_PIPE=>$in1}, {OUT => $in1}],
 
 );
 
