@@ -2,7 +2,7 @@
 =pod
   Unit Tests for decorate
 
-   Copyright (C) 2020 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2020-2021 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -33,7 +33,7 @@ use MIME::Base64 ;
 
 (my $program_name = $0) =~ s|.*/||;
 my $prog = 'decorate';
-
+$prog .= " --sort-cmd=/usr/bin/sort" if $^O eq "netbsd";
 
 # TODO: add localization tests with "grouping"
 # Turn off localization of executable's output.
@@ -222,95 +222,160 @@ C0A81164 C   192.168.17.100
 C0A8110A L   192.168.17.10
 EOF
 
+my $in4=<<'EOF';
+203.0.113.47
+192.0.2.33
+203.0.113.0
+192.0.2.3
+0.0.0.0
+::ffff:192.0.2.42
+2001:db8:6:5:4:3:2:1
+2001:Db8::
+::192.0.2.41
+::1
+::
+EOF
+
+my $out4_ipv6v4map=<<'EOF';
+00000000000000000000FFFFCB00712F 203.0.113.47
+00000000000000000000FFFFC0000221 192.0.2.33
+00000000000000000000FFFFCB007100 203.0.113.0
+00000000000000000000FFFFC0000203 192.0.2.3
+00000000000000000000FFFF00000000 0.0.0.0
+00000000000000000000FFFFC000022A ::ffff:192.0.2.42
+20010DB8000600050004000300020001 2001:db8:6:5:4:3:2:1
+20010DB8000000000000000000000000 2001:Db8::
+000000000000000000000000C0000229 ::192.0.2.41
+00000000000000000000000000000001 ::1
+00000000000000000000000000000000 ::
+EOF
+
+my $out4_ipv6v4comp=<<'EOF';
+000000000000000000000000CB00712F 203.0.113.47
+000000000000000000000000C0000221 192.0.2.33
+000000000000000000000000CB007100 203.0.113.0
+000000000000000000000000C0000203 192.0.2.3
+00000000000000000000000000000000 0.0.0.0
+00000000000000000000FFFFC000022A ::ffff:192.0.2.42
+20010DB8000600050004000300020001 2001:db8:6:5:4:3:2:1
+20010DB8000000000000000000000000 2001:Db8::
+000000000000000000000000C0000229 ::192.0.2.41
+00000000000000000000000000000001 ::1
+00000000000000000000000000000000 ::
+EOF
 
 my @Tests =
 (
- ## basic decoration functions
- ['d1', '--decorate -k1,1:roman' , {IN_PIPE=>$in1}, {OUT => $out1_dec_roman}],
- ['d2', '--decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4}],
- ['d3', '--decorate -k1,1:roman -k2,2:ipv4' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_roman_ipv4}],
- ['d4', '--decorate -k2,2:ipv4 -k1,1:roman' , {IN_PIPE=>$in1},
-  {OUT => $out1_dec_ipv4_roman}],
- ['d5', '--decorate -k1,1:as-is' , {IN_PIPE=>$in1}, {OUT => $out1_asis}],
- ['d6', '--decorate -k1,1:strlen -k2,2:strlen' , {IN_PIPE=>$in1},
-  {OUT => $out1_strlen1}],
- ['d7', '--decorate -k1,1:ipv4inet', {IN_PIPE=>$in2}, {OUT => $out2_ipv4inet}],
- ['d8', '--decorate -k1,1:ipv6',     {IN_PIPE=>$in3}, {OUT => $out3_ipv6}],
- ['d9', '--decorate -k1:strlen' ,    {IN_PIPE=>$in1}, {OUT=>$out1_strlen2}],
- ['d10', '--decorate -k1,1:strlen -k1:strlen' ,  {IN_PIPE=>$in1},
-  {OUT=>$out1_strlen3}],
- ['d11', '--decorate -k1,1:strlen -k2.2,2:strlen' ,  {IN_PIPE=>$in1},
-  {OUT=>$out1_strlen4}],
+  ## basic decoration functions
+  ['d1', '--decorate -k1,1:roman' , {IN_PIPE=>$in1}, {OUT => $out1_dec_roman}],
+  ['d2', '--decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $out1_dec_ipv4}],
+  ['d3', '--decorate -k1,1:roman -k2,2:ipv4' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_roman_ipv4}],
+  ['d4', '--decorate -k2,2:ipv4 -k1,1:roman' , {IN_PIPE=>$in1},
+    {OUT => $out1_dec_ipv4_roman}],
+  ['d5', '--decorate -k1,1:as-is' , {IN_PIPE=>$in1}, {OUT => $out1_asis}],
+  ['d6', '--decorate -k1,1:strlen -k2,2:strlen' , {IN_PIPE=>$in1},
+    {OUT => $out1_strlen1}],
+  ['d7', '--decorate -k1,1:ipv4inet', {IN_PIPE=>$in2}, {OUT => $out2_ipv4inet}],
+  ['d8', '--decorate -k1,1:ipv6',     {IN_PIPE=>$in3}, {OUT => $out3_ipv6}],
+  ['d9', '--decorate -k1:strlen' ,    {IN_PIPE=>$in1}, {OUT=>$out1_strlen2}],
+  ['d10', '--decorate -k1,1:strlen -k1:strlen' ,  {IN_PIPE=>$in1},
+    {OUT=>$out1_strlen3}],
+  ['d11', '--decorate -k1,1:strlen -k2.2,2:strlen' ,  {IN_PIPE=>$in1},
+    {OUT=>$out1_strlen4}],
+  ['d12', '--decorate -k1:ipv6v4map', {IN_PIPE=>$in4}, {OUT=>$out4_ipv6v4map}],
+  ['d13', '--decorate -k1:ipv6v4comp', {IN_PIPE=>$in4},
+    {OUT=>$out4_ipv6v4comp}],
 
- ## basic undecoration
- ['u1', '--undecorate 1' , {IN_PIPE=>$out1_dec_roman}, {OUT => $in1}],
- ['u2', '--undecorate 1' , {IN_PIPE=>$out1_dec_ipv4}, {OUT => $in1}],
- ['u3', '--undecorate 2' , {IN_PIPE=>$out1_dec_roman_ipv4}, {OUT => $in1}],
- ['u4', '--undecorate 2' , {IN_PIPE=>$out1_dec_ipv4_roman}, {OUT => $in1}],
+  ## basic undecoration
+  ['u1', '--undecorate 1' , {IN_PIPE=>$out1_dec_roman}, {OUT => $in1}],
+  ['u2', '--undecorate 1' , {IN_PIPE=>$out1_dec_ipv4}, {OUT => $in1}],
+  ['u3', '--undecorate 2' , {IN_PIPE=>$out1_dec_roman_ipv4}, {OUT => $in1}],
+  ['u4', '--undecorate 2' , {IN_PIPE=>$out1_dec_ipv4_roman}, {OUT => $in1}],
 
- ## sort arguments adjustment
- ['sa1', '--print-sort-args -k1,1:roman', {OUT=>"sort -k1,1\n"}],
- ['sa2', '--print-sort-args -k2,2:roman', {OUT=>"sort -k1,1\n"}],
- ['sa3', '--print-sort-args -k2,2:roman -k4,4n -k1,1:strlen',
-  {OUT=>"sort -k1,1 -k6,6n -k2,2\n"}],
- ['sa4', '--print-sort-args -k2,2:ipv4 -k4,4Vr',{OUT=>"sort -k1,1 -k5,5rV\n"}],
- ['sa5', '--print-sort-args -k2,2:ipv4 -k4r,4V',{OUT=>"sort -k1,1 -k5,5rV\n"}],
- ['sa6', '--print-sort-args -k2,2:ipv4 -k4V,4r',{OUT=>"sort -k1,1 -k5,5rV\n"}],
- ['sa7', '--print-sort-args -k2,2r:ipv4 ',{OUT=>"sort -k1,1r\n"}],
- ['sa8', '--print-sort-args -k2,2r:ipv4 ',{OUT=>"sort -k1,1r\n"}],
- ['sa9', '--print-sort-args -k2,2r:ipv4 -s ',{OUT=>"sort -k1,1r -s\n"}],
- ['sa10', '--print-sort-args -k2,2r:ipv4 -szu ',
-  {OUT=>"sort -k1,1r -s -z -u\n"}],
- ['sa11', '--print-sort-args -k2,2r:ipv4 -S 2G -szu ',
-  {OUT=>"sort -k1,1r -S 2G -s -z -u\n"}],
- ['sa12', '--print-sort-args -k2,2r:ipv4 -S 2G -T /foo/bar -szu ',
-  {OUT=>"sort -k1,1r -S 2G -T /foo/bar -s -z -u\n"}],
- ['sa13', '--print-sort-args -k2,2r:ipv4 -t: -S 2G -T /foo/bar -szu ',
-  {OUT=>"sort -k1,1r -t : -S 2G -T /foo/bar -s -z -u\n"}],
- ['sa14', '--print-sort-args -k2,2r:ipv4 -t: -t:',
-  {OUT=>"sort -k1,1r -t : -t :\n"}],
- ['sa15', '--print-sort-args -k2,2r:ipv4 -t "\\\\0"',
-  {OUT=>"sort -k1,1r -t \\0\n"}],
- ['sa16', '--print-sort-args -k2,2r:ipv4 --compress-program=ZOOP',
-  {OUT=>"sort -k1,1r --compress-program ZOOP\n"}],
- ['sa17', '--print-sort-args -k2,2r:ipv4 --compress-program ZOOP',
-  {OUT=>"sort -k1,1r --compress-program ZOOP\n"}],
- ['sa18', '--print-sort-args -k2,2r:ipv4 --random-source /foo/bar',
-  {OUT=>"sort -k1,1r --random-source /foo/bar\n"}],
- ['sa19', '--print-sort-args -k2,2r:ipv4 --batch-size 9 --parallel=42',
-  {OUT=>"sort -k1,1r --batch-size 9 --parallel 42\n"}],
- ['sa21', '--print-sort-args -k2,2r:ipv4 -c',
-  {OUT=>"sort -k1,1r -c\n"}],
- ['sa22', '--print-sort-args -k2,2r:ipv4 --check',
-  {OUT=>"sort -k1,1r --check\n"}],
- ['sa23', '--print-sort-args -k2,2r:ipv4 -C',
-  {OUT=>"sort -k1,1r -C\n"}],
- ['sa24', '--print-sort-args -k2,2r:ipv4 --check=diagnose-first',
-  {OUT=>"sort -k1,1r --check=diagnose-first\n"}],
- ['sa25', '--print-sort-args -k2,2r:ipv4 --check=foobar12343124321123421432145',
-  {OUT=>"sort -k1,1r --check=foobar12343124321123421432145\n"}],
- ['sa26', '--print-sort-args -k2,2r:ipv4 --check=quiet',
-  {OUT=>"sort -k1,1r --check=quiet\n"}],
- ['sa27', '--print-sort-args -k7 -k2,2r:ipv4 -k5.3,6.2',
-  {OUT=>"sort -k8 -k1,1r -k6.3,7.2\n"}],
+  ## sort arguments adjustment
+  ['sa1', '--print-sort-args -k1,1:roman',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1\n"}],
+  ['sa2', '--print-sort-args -k2,2:roman',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1\n"}],
+  ['sa3', '--print-sort-args -k2,2:roman -k4,4n -k1,1:strlen',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1 -k6,6n -k2,2\n"}],
+  ['sa4', '--print-sort-args -k2,2:ipv4 -k4,4Vr',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1 -k5,5rV\n"}],
+  ['sa5', '--print-sort-args -k2,2:ipv4 -k4r,4V',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1 -k5,5rV\n"}],
+  ['sa6', '--print-sort-args -k2,2:ipv4 -k4V,4r',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1 -k5,5rV\n"}],
+  ['sa7', '--print-sort-args -k2,2r:ipv4 ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r\n"}],
+  ['sa8', '--print-sort-args -k2,2r:ipv4 ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},{OUT=>"sort -k1,1r\n"}],
+  ['sa9', '--print-sort-args -k2,2r:ipv4 -s ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -s\n"}],
+  ['sa10', '--print-sort-args -k2,2r:ipv4 -szu ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -s -z -u\n"}],
+  ['sa11', '--print-sort-args -k2,2r:ipv4 -S 2G -szu ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r -S 2G -s -z -u\n"}],
+  ['sa12', '--print-sort-args -k2,2r:ipv4 -S 2G -T /foo/bar -szu ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r -S 2G -T /foo/bar -s -z -u\n"}],
+  ['sa13', '--print-sort-args -k2,2r:ipv4 -t: -S 2G -T /foo/bar -szu ',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r -t : -S 2G -T /foo/bar -s -z -u\n"}],
+  ['sa14', '--print-sort-args -k2,2r:ipv4 -t: -t:',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -t : -t :\n"}],
+  ['sa15', '--print-sort-args -k2,2r:ipv4 -t "\\\\0"',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -t '\\0'\n"}],
+  ['sa16', '--print-sort-args -k2,2r:ipv4 --compress-program=ZOOP',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r --compress-program ZOOP\n"}],
+  ['sa17', '--print-sort-args -k2,2r:ipv4 --compress-program ZOOP',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r --compress-program ZOOP\n"}],
+  ['sa18', '--print-sort-args -k2,2r:ipv4 --random-source /foo/bar',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r --random-source /foo/bar\n"}],
+  ['sa19', '--print-sort-args -k2,2r:ipv4 --batch-size 9 --parallel=42',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r --batch-size 9 --parallel 42\n"}],
+  ['sa21', '--print-sort-args -k2,2r:ipv4 -c',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -c\n"}],
+  ['sa22', '--print-sort-args -k2,2r:ipv4 --check',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r --check\n"}],
+  ['sa23', '--print-sort-args -k2,2r:ipv4 -C',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'}, {OUT=>"sort -k1,1r -C\n"}],
+  ['sa24', "--print-sort-args -k2,2r:ipv4 '--check=diagnose-first'",
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r '--check=diagnose-first'\n"}],
+  ['sa25', "--print-sort-args -k2,2r:ipv4 " .
+           "--check=foobar12343124321123421432145",
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r '--check=foobar12343124321123421432145'\n"}],
+  ['sa26', '--print-sort-args -k2,2r:ipv4 --check=quiet',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k1,1r '--check=quiet'\n"}],
+  ['sa27', '--print-sort-args -k7 -k2,2r:ipv4 -k5.3,6.2',
+    {OUT_SUBST=>'s/^.*?sort(?: -S)?/sort/'},
+    {OUT=>"sort -k8 -k1,1r -k6.3,7.2\n"}],
 
 
- # Edge Cases
- ['g1', '--decorate -k1,1:roman' , {IN_PIPE=>""}, {OUT => ""}],
+  # Edge Cases
+  ['g1', '--decorate -k1,1:roman' , {IN_PIPE=>""}, {OUT => ""}],
 
- # Header Lines
- ['h1', '-H --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
-  {OUT=>$out1_dec_ipv4_header1}],
- ['h2', '--header 1 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
-  {OUT=>$out1_dec_ipv4_header1}],
- ['h3', '--header=1 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
-  {OUT=>$out1_dec_ipv4_header1}],
- ['h4', '--header 2 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
-  {OUT=>$out1_dec_ipv4_header2}],
- # More header lines than in the input
- ['h5', '--header 8 --decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $in1}],
- ['h6', '--header 9 --decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $in1}],
+  # Header Lines
+  ['h1', '-H --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
+    {OUT=>$out1_dec_ipv4_header1}],
+  ['h2', '--header 1 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
+    {OUT=>$out1_dec_ipv4_header1}],
+  ['h3', '--header=1 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
+    {OUT=>$out1_dec_ipv4_header1}],
+  ['h4', '--header 2 --decorate -k2,2:ipv4', {IN_PIPE=>$in1},
+    {OUT=>$out1_dec_ipv4_header2}],
+  # More header lines than in the input
+  ['h5', '--header 8 --decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $in1}],
+  ['h6', '--header 9 --decorate -k2,2:ipv4' , {IN_PIPE=>$in1}, {OUT => $in1}],
 
   # on a different architecture, would printf(%Lg) print something else?
   # Use OUT_SUBST to trim output to 1.3 digits
